#include "rx32_main.h"

#include "inc\lpt32.h"
#include "inc\function.h"
#include "inc\i2c.h"

unsigned char KSH148_StatusByte = 0;
unsigned KSH_F_Hz = 0, LC72131_F_Hz = 0;

// ----------------------------------------------------------------------------

int KSH148_ReadStatus()
{
   unsigned char x;
   KSH148_StatusByte = 0;

   I2C_Start();
   x = I2C_SendByte(0xC1 + 2 * Form1->AS_ComboBox->ItemIndex); // Address byte
   if(x) return -1;

   x = I2C_ReadByte();
   I2C_Stop();

   KSH148_StatusByte = x;
   return x;
}


unsigned KSH148_SetFreq(unsigned F_Hz, bool ForceReSet = false)
{
   unsigned div, F;
   unsigned char band;

   if(F_Hz == KSH_F_Hz && !ForceReSet) return F_Hz;

   F = F_Hz / 1000;
   div = F / 50;

   F = div * 50;

   band = Form1->BSB_Low;
   if(F >= Form1->LOW_MaxFreq_MaskEdit->Text.ToInt()) band = Form1->BSB_Mid;
   if(F >= Form1->MID_MaxFreq_MaskEdit->Text.ToInt()) band = Form1->BSB_High;

/*  148
   band = 1;                                                        // Low
   if(F >= Form1->LOW_MaxFreq_MaskEdit->Text.ToInt()) band = 2;     // Mid
   if(F >= Form1->MID_MaxFreq_MaskEdit->Text.ToInt()) band = 4;     // High

/*  142
   band = 2;                                                        // Low
   if(F >= Form1->LOW_MaxFreq_MaskEdit->Text.ToInt()) band = 4;     // Mid
   if(F >= Form1->MID_MaxFreq_MaskEdit->Text.ToInt()) band = 1;     // High
*/

   I2C_Start();
   I2C_SendByte(0xC0 + 2 * Form1->AS_ComboBox->ItemIndex); // Address byte
   I2C_SendByte((div >> 8) & 0x7F);                        // Divider byte 1
   I2C_SendByte(div & 0xFF);                               // Divider byte 2
   I2C_SendByte(0x80 | Form1->CB);                         // Control byte
   I2C_SendByte(band);                                     // Band switch byte
   I2C_Stop();

   Form1->StatusBar->Panels->Items[0]->Text = F;
   Form1->KS_StatusRecheck = true;

   KSH_F_Hz = F * 1000;
   return F * 1000;
}



// ==========================================================================


void LC72131_SendBit(bool x)
{
   SetPortBit(Form1->CL_RadioGroup->ItemIndex, 0);   // CLK
   IO_delay();
   SetPortBit(Form1->DI_RadioGroup->ItemIndex, x);   // DATA
   IO_delay();
   SetPortBit(Form1->CL_RadioGroup->ItemIndex, 1);   // CLK
   IO_delay();
}

bool LC72131_InLock()
{
   return ReadPortLine(Form1->DO_RadioGroup->ItemIndex);
}



unsigned LC72131_SetFreq(unsigned F_Hz, bool ForceReSet = false)
{
   unsigned i, k, div, F;
   unsigned steps[]      = { 1000, 3125, 6250, 12500 };   // steps in Hz
   unsigned step_codes[] = { 0x0B, 0x06, 0x05, 0x04  };

   if(F_Hz == LC72131_F_Hz && !ForceReSet) return F_Hz;

   k = Form1->LC72131_Step_ComboBox->ItemIndex;

   div = F_Hz / steps[k];

   SetPortBit(Form1->CE_RadioGroup->ItemIndex, 0);    // CE low
   SetPortBit(Form1->CL_RadioGroup->ItemIndex, 1);    // CLK high
   IO_delay();

   LC72131_SendBit(0);
   LC72131_SendBit(0);
   LC72131_SendBit(0);
   LC72131_SendBit(1);
   LC72131_SendBit(0);
   LC72131_SendBit(1);
   LC72131_SendBit(0);
   LC72131_SendBit(0);

   SetPortBit(Form1->CE_RadioGroup->ItemIndex, 1);    // CE high
   IO_delay();

   if(Form1->LC72131_FMIN_RadioButton->Checked) div = div / 2;

   for(i = 0; i <= 15; i++) LC72131_SendBit((div >> i) & 1);
   LC72131_SendBit(1);                                        // SNS
   LC72131_SendBit(Form1->LC72131_FMIN_RadioButton->Checked); // DVS
   LC72131_SendBit(0);                                        // CTE
   LC72131_SendBit(Form1->XTAL_RadioGroup->ItemIndex);        // XS

   for(i = 0; i < 4; i++) LC72131_SendBit((step_codes[k] >> i) & 1);

   SetPortBit(Form1->CE_RadioGroup->ItemIndex, 0);   // CE low
   IO_delay();

   LC72131_SendBit(1);
   LC72131_SendBit(0);
   LC72131_SendBit(0);
   LC72131_SendBit(1);
   LC72131_SendBit(0);
   LC72131_SendBit(1);
   LC72131_SendBit(0);
   LC72131_SendBit(0);

   SetPortBit(Form1->CE_RadioGroup->ItemIndex, 1);    // CE high
   IO_delay();

   LC72131_SendBit(0);  // IOC1
   LC72131_SendBit(0);  // IOC2
   LC72131_SendBit(0);
   LC72131_SendBit(0);
   LC72131_SendBit(!Form1->BO1_CheckBox->Checked);  // BO1
   LC72131_SendBit(!Form1->BO2_CheckBox->Checked);  // BO2
   LC72131_SendBit(!Form1->BO3_CheckBox->Checked);  // BO3
   LC72131_SendBit(!Form1->BO4_CheckBox->Checked);  // BO4
   LC72131_SendBit(0);  // Don't care
   LC72131_SendBit(1);  // DOC0
   LC72131_SendBit(0);  // DOC1
   LC72131_SendBit(0);  // DOC2
   LC72131_SendBit(0);  // UL0
   LC72131_SendBit(1);  // UL1
   LC72131_SendBit(0);  // DZ0
   LC72131_SendBit(0);  // DZ1
   LC72131_SendBit(0);  // GT0
   LC72131_SendBit(0);  // GT1
   LC72131_SendBit(0);  // TBC
   LC72131_SendBit(0);  // DLC
   LC72131_SendBit(1);  // IFS
   LC72131_SendBit(0);  // 0
   LC72131_SendBit(0);  // 0
   LC72131_SendBit(0);  // 0

   SetPortBit(Form1->CE_RadioGroup->ItemIndex, 0);   // CE
   SetPortBit(Form1->DI_RadioGroup->ItemIndex, 0);   // DATA

   i = 1;
   if(Form1->LC72131_FMIN_RadioButton->Checked) i = 2;

   F = div * i * steps[k];

   Form1->StatusBar->Panels->Items[1]->Text = AnsiString((double) F / 1000);

   if(F != F_Hz) Form1->ShowWarningString("LC72131:   " + AnsiString((double) F_Hz / 1000) + "     !");
   else          Form1->ShowWarningString("");

   LC72131_F_Hz = F;
   return F;
}


// ==========================================================================

void SetFrequency(unsigned F, bool ForceReSet)
{
   unsigned F1, F2, dF;
   int IF1, IF2;

   IF1 = Form1->IF1_MaskEdit->Text.ToInt();
   IF2 = Form1->IF2_MaskEdit->Text.ToInt();

   if(Form1->NoKS)
   {
      F += 1000 * IF2;

      F = LC72131_SetFreq(F) - IF2 * 1000;
      Form1->F = F;
      Form1->StatusBar->Panels->Items[5]->Text = " F (): " + AnsiString(ToMHz(F));
      return;
   }

   F += 1000 * IF1;

   dF = F % 50000;

   F1 = F - dF;
   F2 = (IF1 - IF2) * 1000 - dF;

   F2 = LC72131_SetFreq(F2, ForceReSet);
   F1 =  KSH148_SetFreq(F1, ForceReSet);

   F = F1 - F2 - IF2 * 1000;
   Form1->F = F;
   Form1->StatusBar->Panels->Items[5]->Text = " F (): " + AnsiString(ToMHz(F));
}


bool RX_CarrierDetect()
{
   return ReadPortLine(Form1->CD_RadioGroup->ItemIndex);
}


bool KSH148_InLock()
{
   int a;
   if(Form1->NoKS) return 1;

   a = KSH148_ReadStatus();
   if(a == -1 || !(a & 0x38) || !(a & 0x40)) return 0;
   return 1;
}

