#include "rx32_main.h"
#include "input_form.h"

#include "inc\intrface.h"
#include "inc\function.h"
#include "inc\hardware.h"


int __fastcall FreqListCompare(void *Item1, void *Item2)
{
   double x1 = ((FreqInfo *) Item1) -> F;
   double x2 = ((FreqInfo *) Item2) -> F;

   if(x1 < x2) return -1;
   if(x1 > x2) return 1;
   return 0;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

void __fastcall TForm1::Freq_TabSheetShow(TObject *Sender)
{
   pFreqList->Sort(FreqListCompare);

   Freq_StringGrid->RowCount = pFreqList->Count;

   for(int i = 0; i < pFreqList->Count; i++)
   {
      if(ToHz(((FreqInfo *) pFreqList->Items[i])->F) == F)
      {
         Freq_StringGrid->Row = i;
         break;
      }
   }
   Freq_StringGrid->SetFocus();
}
//---------------------------------------------------------------------------


void __fastcall TForm1::LoadFreqList()
{
   FreqInfo *pFreqInfo;
   TStringList *pSL = new TStringList;
   AnsiString s;
   int i, j;

   pFreqList = new TList;

   ch_ini->ReadSection("Channels", pSL);
   for(i = 0; i < pSL->Count; i++)
   {
      s = ch_ini->ReadString("Channels", pSL->Strings[i], "");

      pFreqInfo = new FreqInfo;

      pFreqInfo->F = ToDouble(pSL->Strings[i]) / 1000000;
      pFreqInfo->Known = false;
      pFreqInfo->Ignore = false;

      for(j = 0; j < 2; j++)
      {
         if(s.Length() && (s[1] == '!' || s[1] == '*'))
         {
            if(s[1] == '*') pFreqInfo->Known  = true;
            else            pFreqInfo->Ignore = true;

            s.Delete(1, 1);
         }
      }
      while(s.Length() && s[1] == ' ') s.Delete(1, 1);
      pFreqInfo->Comment = s;

      pFreqList->Add(pFreqInfo);
   }
//   pFreqList->Sort(FreqListCompare);
//   Freq_StringGrid->RowCount = pFreqList->Count;

   delete pSL;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::UnLoadFreqList()
{
   AnsiString s;
   FreqInfo *pFreqInfo;

   if(FreqListSave_CheckBox->Checked) ch_ini->EraseSection("Channels");

   for(int i = 0; i < pFreqList->Count; i++)
   {
      pFreqInfo = (FreqInfo *) pFreqList->Items[i];

      s =  (pFreqInfo->Known)  ? "*" : " ";
      s += (pFreqInfo->Ignore) ? "!" : " ";
      s += " ";
      s += pFreqInfo->Comment;

      if(FreqListSave_CheckBox->Checked) ch_ini->WriteString("Channels", AnsiString(1000000 * pFreqInfo->F), s);
      delete pFreqInfo;
   }
   delete pFreqList;
   pFreqList = NULL;
}


//---------------------------------------------------------------------------

void __fastcall TForm1::Add_SpeedButtonClick(TObject *Sender)
{
   double f = 0;
   FreqInfo *pFreqInfo;
   int i;

   if(InputDoubleDialog(" ", "   ", f, false))
   {
      if(GetFreqInfo(f))
      {
         ShowMessage(MSG_WARNING, "   !");
         return;
      }

      pFreqInfo = new FreqInfo;
      pFreqInfo->F = f;
      pFreqInfo->Known = false;
      pFreqInfo->Ignore = false;
      InputDialog("", "   ", pFreqInfo->Comment);

      pFreqList->Add(pFreqInfo);
      pFreqList->Sort(FreqListCompare);
      i = pFreqList->IndexOf(pFreqInfo);

      Freq_StringGrid->RowCount = pFreqList->Count;

      Freq_StringGrid->Repaint();
      Freq_StringGrid->Col = 0;
      Freq_StringGrid->Row = i;
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::DelRow_SpeedButtonClick(TObject *Sender)
{
   int i = Freq_StringGrid->Row;

   if(!pFreqList->Count) return;

   FreqInfo *pFreqInfo = (FreqInfo *) pFreqList->Items[i];

   if(MessageDlg(AnsiString(pFreqInfo->F) + " \n ?", mtConfirmation, TMsgDlgButtons() << mbYes << mbNo, 0) == mrYes)
   {
      if(pFreqList->Count > i)
      {
         delete pFreqInfo;
         pFreqList->Delete(i);

         Freq_StringGrid->RowCount = pFreqList->Count;
         Freq_StringGrid->Col = 0;
         Freq_StringGrid->Repaint();
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Freq_StringGridDblClick(TObject *Sender)
{
   if(!pFreqList->Count) return;

   FreqInfo *pFreqInfo = (FreqInfo *) pFreqList->Items[Freq_StringGrid->Row];

   switch(Freq_StringGrid->Col)
   {
      case 0: if(!ScanActive)
              {
                 F = ToHz(pFreqInfo->F);
                 SetFrequency(F);
              }
              break;

      case 1: pFreqInfo->Known ^= 1;
              break;

      case 2: pFreqInfo->Ignore ^= 1;
              break;

      case 3: InputDialog("", "   ", pFreqInfo->Comment);
              break;
   }
   Freq_StringGrid->Repaint();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Freq_StringGridMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   int x1 = 0;

   if(X < (x1 += Freq_StringGrid->ColWidths[0]))
   {
      Freq_StringGrid->Hint = ", ";
      return;
   }
   if(X < (x1 += Freq_StringGrid->ColWidths[1]))
   {
      Freq_StringGrid->Hint = "\"\" ";
      return;
   }
   if(X < (x1 += Freq_StringGrid->ColWidths[2]))
   {
      Freq_StringGrid->Hint = "  ";
      return;
   }
   if(X < (x1 += Freq_StringGrid->ColWidths[3]))
   {
      Freq_StringGrid->Hint = "  ";
      return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Freq_StringGridDrawCell(TObject *Sender, int ACol,
      int ARow, TRect &Rect, TGridDrawState State)
{
   if(!pFreqList->Count) return;

   FreqInfo *pFreqInfo = (FreqInfo *) pFreqList->Items[ARow];

   if(State.Contains(gdSelected))
   {
      Freq_StringGrid->Canvas->Font->Color = (ToHz(pFreqInfo->F) == F) ? clLime : clWindow;
   }
   else
   {
      Freq_StringGrid->Canvas->Font->Color = (ToHz(pFreqInfo->F) == F) ? clGreen : clWindowText;
   }

   switch(ACol)
   {
      case 0: Freq_StringGrid->Canvas->TextOut(Rect.Left + 1, Rect.Top + 2, pFreqInfo->F);
              break;

      case 1: if(pFreqInfo->Known)  ImageList->Draw(Freq_StringGrid->Canvas, Rect.Left - 1, Rect.Top, 0, true);
              break;

      case 2: if(pFreqInfo->Ignore) ImageList->Draw(Freq_StringGrid->Canvas, Rect.Left + 1, Rect.Top, 5, true);
              break;

      case 3: Freq_StringGrid->Canvas->TextOut(Rect.Left + 1, Rect.Top + 2, pFreqInfo->Comment);

   }
}
//---------------------------------------------------------------------------

FreqInfo * TForm1::GetFreqInfo(double F)
{
   FreqInfo *pFreqInfo;

   for(int i = 0; i < pFreqList->Count; i++)
   {
      pFreqInfo = (FreqInfo *) pFreqList->Items[i];
      if(pFreqInfo->F == F) return pFreqInfo;
   }
   return NULL;
}

//---------------------------------------------------------------------------

void __fastcall TForm1::SetFreqInfo(double F, bool Known, bool Ignore, AnsiString Comment)
{
   FreqInfo *pFI = GetFreqInfo(F);
   bool flag = false;

   if(!pFI)
   {
      flag = true;

      pFI = new FreqInfo;
      pFI->F = F;
   }
   pFI->Comment = Comment;
   pFI->Known   = Known;
   pFI->Ignore  = Ignore;

   if(flag)
   {
      pFreqList->Add(pFI);
      pFreqList->Sort(FreqListCompare);
      Freq_StringGrid->Repaint();
   }
}    
