//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "rx32_main.h"
#include "scan.h"
#include "freq_form.h"

#include "inc\lpt32.h"
#include "inc\intrface.h"
#include "inc\function.h"
#include "inc\hardware.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TScanForm *ScanForm;
//---------------------------------------------------------------------------
__fastcall TScanForm::TScanForm(TComponent* Owner)
        : TForm(Owner)
{
   pList = new TList;
}

//---------------------------------------------------------------------------
void __fastcall TScanForm::FormDestroy(TObject *Sender)
{
   for(int i = 0; i < pList->Count; i++) delete pList->Items[i];
   delete pList;
}
//---------------------------------------------------------------------------


void __fastcall TScanForm::AddScanInterval(unsigned f_min, unsigned f_max, unsigned f_step)
{
   ScanInterval *pSI = new ScanInterval;

   pSI->Fmin = f_min;
   pSI->Fmax = f_max;
   pSI->Step = f_step;

   pList->Add(pSI);
}

//---------------------------------------------------------------------------

void __fastcall TScanForm::Execute()
{
   int i, j, k, t, SQL_T;
   unsigned Fmin, Fmax, Step, f, pass, ignored;
   ScanInterval *pSI;
   FreqInfo *pFI;
   TStringList *pIgnoreNowList;
   AnsiString s; //, sf;
   TDateTime T0 = Now();
   bool flag;

   Show();

   ScanFlag = true;
   ScanPaused = false;

   pass = 0;
   ignored = 0;
   t = Form1->PLLTimeout_MaskEdit->Text.ToInt();
   SQL_T = Form1->SQLWait_MaskEdit->Text.ToInt();

   Form1->Mute = true;
   Form1->ScanActive = true;
   delay_ms(50);


   pIgnoreNowList = new TStringList;

   while(ScanFlag)
   {
      pass++;
      Pass_Label->Caption = pass;

      for(i = 0; i < pList->Count && ScanFlag; i++)
      {
         pSI = (ScanInterval *) pList->Items[i];

         Fmin = pSI->Fmin;
         Fmax = pSI->Fmax;
         Step = pSI->Step;

         if(Fmax && Step)
         {
            s.printf("%.9g - %.9g  ( %g )", ToMHz(Fmin), ToMHz(Fmax), TokHz(Step));
            flag = false;
         }
         else
         {
            s.printf("%.9g", ToMHz(Fmin));
            flag = true;
         }

         SI_Label->Caption = s;
         SI_Label->Left = (ScanInfo_GroupBox->Width - SI_Label->Width) / 2;

         ignored = 0;

         for(f = Fmin; (f <= Fmax && ScanFlag) || flag; f += Step, flag = false)
         {
            Ignore_Count_Label->Caption = ignored;

            if(!Form1->NotUseIgnoreList_CheckBox->Checked && Form1->IgnoreList_ListBox->Items->IndexOf(ToMHz(f)) > -1)
            {
               ignored++;
               continue;
            }
            if(pIgnoreNowList->IndexOf(AnsiString(f)) != -1)
            {
               ignored++;
               continue;
            }

            pFI = Form1->GetFreqInfo(ToMHz(f));

            if(pFI)
            {
               if(( Form1->IgnoreKnown_CheckBox     ->Checked && pFI->Known) ||
                  (!Form1->NotUseIgnoreFlag_CheckBox->Checked && pFI->Ignore))
               {
                  ignored++;
                  continue;
               }
            }
          //
            SetFrequency(f);
          //

            ScanF_MaskEdit->Text = f;
            if(ScanF_MaskEdit->Focused()) ScanStop_BitBtn->SetFocus();

            Scan_ProgressBar->Position = ((double) (f - Fmin) / (Fmax - Fmin)) * 1000;
            ScanTime_Label->Caption = TimeToStr(Now() - T0);

            Application->ProcessMessages();

            k = 0;
            while(k < t && (!LC72131_InLock() || !KSH148_InLock()))
            {
               delay_ms(1);
               k++;
               if(k == t) ShowMessage(MSG_ERROR, "  !");
            }
            if(k == t)
            {
               ScanStop_BitBtnClick(this);
               break;
            }
            if(SQL_T) delay_ms(SQL_T);


            if(RX_CarrierDetect())
            {
               for(k = 0; k < 15; k++)
               {
                  delay_ms(2);
                  if(!RX_CarrierDetect()) break;
               }
               if(k < 15) continue;

               
               Form1->Mute = false;
               Form1->ScanActive = false;
               Form1->Control_TabSheet->Enabled = true;

               TFreqForm *p = new TFreqForm(this);
               this->Enabled = false;
               while(1)
               {
                  k = p->Execute(f);

                  if(k >= 20)
                  {
                     k -= 20;
                     pIgnoreNowList->Add(AnsiString(f));
                  }
                  if(k < 10) break;
                  f += Step * (k - 11);
                  SetFrequency(f);
               }
               this->Enabled = true;
               delete p;

               if(k)
               {
                  ScanStop_BitBtnClick(this);
                  break;
               }

               Form1->Mute = true;
               Form1->ScanActive = true;
               Form1->Control_TabSheet->Enabled = false;
               delay_ms(50);

            }
         }
      }
   }

   Form1->Mute = false;
   Form1->ScanActive = false;
}
//---------------------------------------------------------------------------

void __fastcall TScanForm::ScanStop_BitBtnClick(TObject *Sender)
{
   ScanFlag = false;
   ScanPaused = false;
}
//---------------------------------------------------------------------------

